@@ -316,7 +316,7 @@ on_static_display_status_changed (GdmDisplay             *display,
                 factory->priv->num_failures++;
                 if (factory->priv->num_failures > MAX_DISPLAY_FAILURES) {
                         /* oh shit */
-                        g_critical ("GdmLocalDisplayFactory: maximum number of displays failures reached: check Xorg.log for errors");
+                        g_warning ("GdmLocalDisplayFactory: maximum number of X display failures reached: check X server log for errors");
                         exit (1);
                 }
 
@@ -324,6 +324,8 @@ on_static_display_status_changed (GdmDisplay             *display,
                 break;
         case GDM_DISPLAY_UNMANAGED:
                 break;
+        case GDM_DISPLAY_PREPARED:
+                break;
         case GDM_DISPLAY_MANAGED:
                 break;
         default:
